@@ -1,7 +1,7 @@
 /*
  * Main include file for Mathomatic.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #define	true	1
@@ -25,7 +25,7 @@
 #define	isfinite(d)	finite(d)			/* true if d is finite (not infinity or NaN) */
 #endif
 
-#define	ARR_CNT(a)	(sizeof(a)/sizeof(a[0]))	/* returns the number of elements in an array */
+#define	ARR_CNT(a)	((int) (sizeof(a)/sizeof(a[0])))	/* returns the number of elements in an array */
 
 #define	MAX_K_INTEGER	1.0e14				/* maximum representable integer, 14 digits for doubles */
 #define	MAX_PRECISION	16				/* maximum useful display precision (number of digits) */
@@ -45,7 +45,7 @@
 #define	TMP_FILE	"/tmp/mathomatic.XXXXXX"	/* temp file template */
 
 #define	PROMPT		"-> "				/* user interface prompt string */
-#define	HTML_PROMPT	"-&gt; "
+#define	HTML_PROMPT	"&mdash;&gt; "
 
 #define	MAX_CMD_LEN	PATH_MAX			/* maximum command line length (not equations), also max file name length */
 #define	MAX_PROMPT_LEN	STANDARD_SCREEN_COLUMNS		/* maximum length of prompts */
@@ -63,19 +63,20 @@
 
 /*
  * The following defines the default maximum mathematical expression size.
- * Expression arrays are allocated with this size by default.
- * It is linearly related to the actual memory usage of Mathomatic.
- * This should be made much smaller for Palmtops or embedded systems.
+ * Expression arrays are allocated with this size by default,
+ * there are 2 of these for each equation space (1 for LHS and 1 for RHS).
+ * DEFAULT_N_TOKENS is linearly related to the actual memory usage of Mathomatic.
+ * This should be made much smaller for handhelds and embedded systems.
  * Do not set to less than 100.
  */
 #define	DEFAULT_N_TOKENS	30000
 
 #define	DIVISOR_SIZE	(DEFAULT_N_TOKENS / 2)	/* a nice maximum divisor size */
 
-#define	MAX_VAR_NAMES	8000			/* maximum number of long variable names, keep this under (VAR_MASK - VAR_OFFSET) */
-#define	MAX_VAR_LEN	100			/* maximum size of long variable names */
+#define	MAX_VAR_NAMES	8000			/* maximum number of variable names, keep this under (VAR_MASK - VAR_OFFSET) */
+#define	MAX_VAR_LEN	100			/* maximum number of characters in variable names */
 
-#define	MAX_VARS	500			/* maximum number of unique variables handled in each equation */
+#define	MAX_VARS	min(DEFAULT_N_TOKENS / 4, 1000)	/* maximum number of unique variables handled in each equation */
 
 #define	VAR_OFFSET	'A'			/* makes space for predefined variables */
 #define	VAR_MASK	0x3fffL			/* mask for bits containing a reference to the variable name */
@@ -97,7 +98,7 @@ typedef union {
 /* predefined special variables follow (order is important) */
 #define	V_NULL		0L	/* null variable (display nothing) */
 #define	V_E		1L	/* the constant "e" or "e#" */
-#define	V_PI		2L	/* the constant "pi" */
+#define	V_PI		2L	/* the constant "pi" or "pi#" */
 #define	IMAGINARY	3L	/* the imaginary constant "i" or "i#" */
 #define	SIGN		4L	/* for "sign" variables */
 #define	MATCH_ANY	5L	/* match any variable (wild-card variable) */
@@ -131,7 +132,7 @@ typedef struct {		/* variable sort data structure */
 #define	side_debug(level, p1, n1)
 #define debug_string(level, str)
 #else
-#define	error(str)			{ printf("%s\n", str); }
+#define	error(str)			{ error_str = str; printf("%s\n", str); }
 #define list_tdebug(level)		list_debug(level, tlhs, n_tlhs, trhs, n_trhs)
 #define	side_debug(level, p1, n1)	list_debug(level, p1, n1, NULL, 0)
 #define debug_string(level, str)	{ if (debug_level >= (level)) printf("%s\n", str); }
