@@ -1,7 +1,7 @@
 /*
  * Mathomatic commands that don't belong anywhere else.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -13,14 +13,22 @@ static int	complex_func();
 static int	elim_sub();
 #if	!LIBRARY
 static int	edit_sub();
-static int	find_more();
-static int	opt_es();
 
 /* Global variables for the optimize command. */
 static int	opt_en[N_EQUATIONS];
 static int	last_temp_var = 0;
 #endif
 
+/*
+ * Compare function for qsort(3).
+ */
+static int
+vcmp(p1, p2)
+sort_type	*p1, *p2;
+{
+	return(p2->count - p1->count);
+}
+
 /*
  * The version command.
  */
@@ -41,7 +49,7 @@ char	*cp;	/* the command line argument */
 int
 version_report()
 {
-	fprintf(gfp, _("Mathomatic version %s\nCompile flags used: "), VERSION);
+	fprintf(gfp, _("Mathomatic version %s, written by George Gesslein II.\nCompile flags used: "), VERSION);
 #if	__STDC__
 	fprintf(gfp, "__STDC__ ");
 #endif
@@ -282,7 +290,7 @@ int	product_flag;	/* true for product, otherwise sum */
  */
 static int
 find_more(equation, np, en)
-token_type	*equation;	/* expression pointer */
+token_type	*equation;	/* expression to search */
 int		*np;		/* pointer to length of expression */
 int		en;		/* equation space number */
 {
@@ -291,7 +299,7 @@ int		en;		/* equation space number */
 	int	diff_sign;
 	int	found_se;	/* found sub-expression flag */
 
-	if (!solved_equation(en)) {
+	if (*np <= 0 || !solved_equation(en)) {
 		return false;
 	}
 	for (level = 1, found_se = true; found_se; level++) {
@@ -348,6 +356,9 @@ int		*np;
 	long	v;
 	char	temp_buf[50];
 
+	if (*np <= 0) {
+		return false;
+	}
 	for (level = 1, found_se = true; found_se; level++) {
 		for (i = 1, found_se = false; i < *np; i = j + 2) {
 			for (j = i; j < *np && equation[j].level > level; j += 2)
@@ -439,7 +450,7 @@ char	*cp;
 {
 	int	i, j, k, i1;
 	int	start, stop;
-	int	rv = false;
+	int	rv = false, flag;
 	int	start_en;
 
 	if (!get_range_eol(&cp, &start, &stop)) {
@@ -447,28 +458,42 @@ char	*cp;
 	}
 	opt_en[0] = -1;
 	start_en = 0;
-	for (i = start; i <= stop; i++) {
-		simp_sub(i);
+	for (j = i = start; i <= stop; i++) {
+		if (n_lhs[i]) {
+			j = i;
+			simp_sub(i);
+		}
 	}
-	for (i = start; i <= stop; i++) {
-		for (j = start; j <= stop; j++) {
-			if (i != j) {
-				while (find_more(rhs[i], &n_rhs[i], j)) {
-					rv = true;
+	stop = j;
+	do {
+		flag = false;
+		for (i = start; i <= stop; i++) {
+			for (j = start; j <= stop; j++) {
+				if (i != j) {
+					while (find_more(rhs[i], &n_rhs[i], j)) {
+						flag = true;
+						rv = true;
+					}
 				}
 			}
 		}
-	}
+	} while (flag);
 	for (i = start; i <= stop; i++) {
 		if (n_lhs[i] == 0)
 			continue;
-		for (j = 0; opt_en[j] >= 0; j++) {
-			simp_sub(opt_en[j]);
-			while (find_more(lhs[i], &n_lhs[i], opt_en[j]))
-				;
-			while (find_more(rhs[i], &n_rhs[i], opt_en[j]))
-				;
-		}
+		do {
+			flag = false;
+			simp_sub(i);
+			for (j = 0; opt_en[j] >= 0; j++) {
+				simp_sub(opt_en[j]);
+				if (i != opt_en[j]) {
+					while (find_more(lhs[i], &n_lhs[i], opt_en[j]))
+						flag = true;
+					while (find_more(rhs[i], &n_rhs[i], opt_en[j]))
+						flag = true;
+				}
+			}
+		} while (flag);
 		while (opt_es(lhs[i], &n_lhs[i])) {
 			rv = true;
 		}
@@ -545,34 +570,28 @@ int	en;	/* equation space number */
 #endif
 
 /*
- * The set command.
+ * Display current working directory.
+ *
+ * Return true if successful.
  */
 int
-set_cmd(cp)
-char	*cp;
+display_current_directory()
 {
+#if	!SECURE
 	char	buf[MAX_CMD_LEN];
 
-	if (*cp == '\0') {
-		fprintf(gfp, _("Options are set as follows:\n\n"));
-
-		output_options();
-
-		fprintf(gfp, "columns = %d\n", screen_columns);
-
-#if	!SECURE
-		if (getcwd(buf, sizeof(buf))) {
-			fprintf(gfp, "directory = %s\n", buf);
-		}
-#endif
+	if (getcwd(buf, sizeof(buf))) {
+		fprintf(gfp, "directory = %s\n", buf);
 		return true;
 	}
-	return set_options(cp);
+#endif
+	return false;
 }
 
 /*
  * Output the current set options in a format suitable for RC_FILE.
  */
+void
 output_options()
 {
 	if (!autosolve) {
@@ -619,16 +638,18 @@ output_options()
 	}
 	fprintf(gfp, "true_modulus\n");
 
-	if (!finance_option) {
-		fprintf(gfp, "no ");
-	}
-	fprintf(gfp, "finance\n");
+	fprintf(gfp, "finance = %d\n", finance_option);
 
 	if (!factor_int_flag) {
 		fprintf(gfp, "no ");
 	}
 	fprintf(gfp, "factor_integers\n");
 
+	if (!right_associative_power) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "right_associative_power\n");
+
 	fprintf(gfp, "special_variable_character = %c\n", special_variable_character);
 }
 
@@ -672,10 +693,17 @@ next_option:
 #if	!SECURE
 	if (strncasecmp(cp, "dir", 3) == 0) {
 		cp = skip_param(cp);
+		if (*cp == '\0') {
+			cp1 = getenv("HOME");
+			if (cp1) {
+				cp = cp1;
+			}
+		}
 		if (chdir(cp)) {
 			error(_("Error changing directory."));
 			return false;
 		}
+		display_current_directory();
 		return true;
 	}
 #endif
@@ -738,7 +766,7 @@ next_option:
 		case_sensitive_flag = !negate;
 		goto next_option;
 	}
-	if (strncasecmp(cp, "display2d", 9) == 0) {
+	if (strncasecmp(cp, "display2d", 7) == 0) {
 		cp = skip_param(cp);
 		display2d = !negate;
 		goto next_option;
@@ -748,7 +776,7 @@ next_option:
 		preserve_roots = !negate;
 		goto next_option;
 	}
-	if (strncasecmp(cp, "true", 4) == 0) {
+	if (strncasecmp(cp, "true_modulus", 4) == 0) {
 		cp = skip_param(cp);
 		true_modulus = !negate;
 		goto next_option;
@@ -767,18 +795,56 @@ next_option:
 	}
 	if (strncasecmp(cp, "finance", 7) == 0) {
 		cp = skip_param(cp);
-		finance_option = !negate;
+		if (negate) {
+			finance_option = 0;
+		} else {
+			i = decstrtol(cp, &cp1);
+			if (cp == cp1) {
+				i = 2;
+			}
+			if (i > 50) {
+				error(_("Maximum is 50."));
+				return false;
+			}
+			cp = cp1;
+			finance_option = i;
+		}
 		goto next_option;
 	}
-	if (strncasecmp(cp, "factor", 6) == 0) {
+	if (strncasecmp(cp, "factor_integers", 6) == 0) {
 		cp = skip_param(cp);
 		factor_int_flag = !negate;
 		goto next_option;
 	}
+	if (strncasecmp(cp, "right_associative_power", 5) == 0) {
+		cp = skip_param(cp);
+		right_associative_power = !negate;
+		goto next_option;
+	}
 	error(_("Unknown set option."));
 	return false;
 }
 
+/*
+ * The set command.
+ */
+int
+set_cmd(cp)
+char	*cp;
+{
+	if (*cp == '\0') {
+		fprintf(gfp, _("Options are set as follows:\n\n"));
+
+		output_options();
+
+		fprintf(gfp, "columns = %d\n", screen_columns);
+
+		display_current_directory();
+		return true;
+	}
+	return set_options(cp);
+}
+
 /*
  * The echo command.
  */
@@ -968,9 +1034,15 @@ char	*cp;
 {
 	int	i;
 	double	count;
+	int	arithmetic_mean = false;
 
-	if (extra_characters(cp))
+	if (strncasecmp(cp, "average", 4) == 0) {
+		arithmetic_mean = true;
+		cp = skip_param(cp);
+	}
+	if (extra_characters(cp)) {
 		return false;
+	}
 	trhs[0] = zero_token;
 	n_trhs = 1;
 	for (count = 0.0;; count++) {
@@ -978,28 +1050,31 @@ char	*cp;
 		list_proc(trhs, n_trhs, false);
 		fprintf(gfp, "\n");
 		if (count > 0.0) {
-			/* calculate and display the average */
-			blt(tlhs, trhs, n_trhs * sizeof(token_type));
-			n_tlhs = n_trhs;
-			if ((n_tlhs + 2) > n_tokens) {
-				error_huge();
+			if (arithmetic_mean) {
+				/* calculate and display the average */
+				blt(tlhs, trhs, n_trhs * sizeof(token_type));
+				n_tlhs = n_trhs;
+				if ((n_tlhs + 2) > n_tokens) {
+					error_huge();
+				}
+				for (i = 0; i < n_tlhs; i++) {
+					tlhs[i].level++;
+				}
+				tlhs[n_tlhs].kind = OPERATOR;
+				tlhs[n_tlhs].level = 1;
+				tlhs[n_tlhs].token.operatr = DIVIDE;
+				n_tlhs++;
+				tlhs[n_tlhs].kind = CONSTANT;
+				tlhs[n_tlhs].level = 1;
+				tlhs[n_tlhs].token.constant = count;
+				n_tlhs++;
+				calc_simp(tlhs, &n_tlhs);
+				fprintf(gfp, _("Average = "));
+				list_proc(tlhs, n_tlhs, false);
+				fprintf(gfp, "\n");
 			}
-			for (i = 0; i < n_tlhs; i++) {
-				tlhs[i].level++;
-			}
-			tlhs[n_tlhs].kind = OPERATOR;
-			tlhs[n_tlhs].level = 1;
-			tlhs[n_tlhs].token.operatr = DIVIDE;
-			n_tlhs++;
-			tlhs[n_tlhs].kind = CONSTANT;
-			tlhs[n_tlhs].level = 1;
-			tlhs[n_tlhs].token.constant = count;
-			n_tlhs++;
-			calc_simp(tlhs, &n_tlhs);
-			fprintf(gfp, _("Average = "));
-			list_proc(tlhs, n_tlhs, false);
-			fprintf(gfp, "\n");
 		}
+		fprintf(gfp, "\n");
 		my_strlcpy(prompt_str, _("Enter value: "), sizeof(prompt_str));
 		if (!get_expr(tlhs, &n_tlhs)) {
 			break;
@@ -1021,6 +1096,7 @@ char	*cp;
 		n_trhs += n_tlhs;
 		calc_simp(trhs, &n_trhs);
 	}
+	fprintf(gfp, _("End.\n"));
 	return true;
 }
 #endif
@@ -1045,10 +1121,23 @@ char	*cp;
 	token_type	*source;
 	int		n;
 	int		diff_sign;
+	char		buf[MAX_CMD_LEN];
+	int		factor_flag = false, repeat_flag = false, value_entered;
 
 	if (current_not_defined()) {
 		return false;
 	}
+	for (;; cp = skip_param(cp)) {
+		if (strcmp_tospace(cp, "factor") == 0) {
+			factor_flag = true;
+			continue;
+		}
+		if (strcmp_tospace(cp, "repeat") == 0) {
+			repeat_flag = true;
+			continue;
+		}
+		break;
+	}
 	i = cur_equation;
 	if (n_rhs[i]) {
 		source = rhs[i];
@@ -1062,18 +1151,23 @@ char	*cp;
 		if (cp == NULL) {
 			return false;
 		}
+		if (!found_var(source, n, it_v)) {
+			error(_("Specified iteration variable not found."));
+			return false;
+		}
+		if (*cp == '\0') {
+			my_strlcpy(prompt_str, _("Enter number of iterations: "), sizeof(prompt_str));
+			if ((cp = get_string(buf, sizeof(buf))) == NULL)
+				return false;
+		}
 		iterations = decstrtol(cp, &cp);
 		if (*cp || iterations <= 0) {
 			error(_("Invalid number of iterations."));
 			return false;
 		}
 	}
-	if (it_v) {
-		if (!found_var(source, n, it_v)) {
-			error(_("Iteration variable not found."));
-			return false;
-		}
-	}
+calc_again:
+	value_entered = false;
 	n_trhs = n;
 	blt(trhs, source, n_trhs * sizeof(token_type));
 	last_v = 0;
@@ -1095,8 +1189,12 @@ char	*cp;
 		list_var(v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("Enter %s: "), var_str);
 		if (!get_expr(tlhs, &n_tlhs)) {
+			if (repeat_flag) {
+				return true;
+			}
 			continue;
 		}
+		value_entered = true;
 		/* disguise all variables in the entered expression by making them negative: */
 		for (j = 0; j < n_tlhs; j += 2)
 			if (tlhs[j].kind == VARIABLE)
@@ -1110,8 +1208,12 @@ char	*cp;
 	if (it_v) {
 		list_var(it_v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("Enter initial %s: "), var_str);
-		while (!get_expr(tes, &n_tes))
-			;
+		while (!get_expr(tes, &n_tes)) {
+			if (repeat_flag) {
+				return true;
+			}
+		}
+		value_entered = true;
 		calc_simp(tes, &n_tes);
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
@@ -1121,15 +1223,15 @@ char	*cp;
 			subst_var_with_exp(trhs, &n_trhs, tes, n_tes, it_v);
 			calc_simp(trhs, &n_trhs);
 			if (se_compare(trhs, n_trhs, tes, n_tes, &diff_sign) && !diff_sign) {
-				printf(_("Convergence reached after %ld iterations.\n"), l + 1);
+				fprintf(gfp, _("Convergence reached after %ld iterations.\n"), l + 1);
 				break;
 			}
 			blt(tes, trhs, n_trhs * sizeof(token_type));
 			n_tes = n_trhs;
 		}
-	} else {
-		calc_simp(trhs, &n_trhs);
 	}
+	calc_simp(trhs, &n_trhs);
+	fprintf(gfp, "\n");
 	for (j = 0; j < ARR_CNT(sa_mark); j++)
 		sa_mark[j] = false;
 	for (j = 0; j < n_trhs; j += 2) {
@@ -1183,17 +1285,19 @@ char	*cp;
 		if (k)
 			fprintf(gfp, ":\n");
 		calc_simp(tlhs, &n_tlhs);
-		if (factor_int_flag) {
-			factor_int(tlhs, &n_tlhs);
+		if (factor_flag) {
+			simp_side(tlhs, &n_tlhs);
 		}
 		fprintf(gfp, " ");
 		if (n_rhs[i]) {
 			list_proc(lhs[i], n_lhs[i], false);
 			fprintf(gfp, " = ");
 		}
-		list_proc(tlhs, n_tlhs, false);
+		list_fact(tlhs, &n_tlhs, factor_flag);
 		fprintf(gfp, "\n\n");
 	}
+	if (value_entered && repeat_flag)
+		goto calc_again;
 	return true;
 }
 #endif
@@ -1275,7 +1379,6 @@ char	*cp;
 {
 	int		i, j;
 	int		diff_sign;
-	int		already_solved;
 
 	i = decstrtol(cp, &cp) - 1;
 	if (not_defined(i)) {
@@ -1327,8 +1430,7 @@ char	*cp;
 		error(_("Cannot compare an expression with an equation."));
 		return false;
 	}
-	already_solved = (solved_equation(i) && solved_equation(j));
-	if (already_solved) {
+	if (solved_equation(i) && solved_equation(j)) {
 		simp_loop(rhs[i], &n_rhs[i]);
 		simp_loop(rhs[j], &n_rhs[j]);
 		if (compare_rhs(i, j, &diff_sign)) {
@@ -1446,8 +1548,18 @@ char	*cp;
 			fprintf(gfp, _("No GCD found.\n"));
 			return true;
 		}
-		fprintf(gfp, _("Greatest Common Divisor (GCD) = %.*g\n"), precision, d3);
-		fprintf(gfp, _("Least Common Multiple (LCM) = %.*g\n"), precision, (d1 * d2) / d3);
+		if (factor_int_flag && factor_one(d3)) {
+			fprintf(gfp, "GCD = ");
+			display_unique();
+		} else {
+			fprintf(gfp, "GCD = %.*g\n", precision, d3);
+		}
+		if (factor_int_flag && factor_one((d1 * d2) / d3)) {
+			fprintf(gfp, "LCM = ");
+			display_unique();
+		} else {
+			fprintf(gfp, "LCM = %.*g\n", precision, (d1 * d2) / d3);
+		}
 		return true;
 	}
 	if (parse_complex(rhs[i], nr, &c1) && parse_complex(lhs[i], nl, &c2)) {
@@ -1463,9 +1575,11 @@ char	*cp;
 		list_var(v_tmp, 0);
 		fprintf(gfp, _("Polynomial division successful using base variable (%s).\n"), var_str);
 		fprintf(gfp, _("The quotient is:\n"));
-		list_proc(tlhs, n_tlhs, false);
+		es_make_fractions_and_group(tlhs, &n_tlhs);
+		list_fact(tlhs, &n_tlhs, false);
 		fprintf(gfp, _("\n\nThe remainder is:\n"));
-		list_proc(trhs, n_trhs, false);
+		es_make_fractions_and_group(trhs, &n_trhs);
+		list_fact(trhs, &n_trhs, false);
 		fprintf(gfp, "\n");
 	} else {
 		fprintf(gfp, _("Polynomial division failed.\n"));
@@ -1478,7 +1592,8 @@ char	*cp;
 	if (j) {
 		simp_divide(trhs, &n_trhs);
 		fprintf(gfp, _("Polynomial GCD (Euclidean algorithm iterations = %d):\n"), j);
-		list_proc(trhs, n_trhs, false);
+		es_make_fractions_and_group(trhs, &n_trhs);
+		list_fact(trhs, &n_trhs, false);
 		fprintf(gfp, "\n");
 	} else {
 		fprintf(gfp, _("No univariate polynomial GCD found.\n"));
@@ -1496,9 +1611,10 @@ char	*cp;
 {
 	long	v, last_v, v1, va[MAX_VARS];
 	int	i, n;
-	int	using_flag, did_something = false;
+	int	success_flag = false, did_something = false, repeat_flag = false;
 	char	used[N_EQUATIONS];
 	int	vc = 0;
+	char	*cp_start;
 
 	if (*cp == '\0') {
 		error(_("Please specify the variables to eliminate or \"all\" for all variables."));
@@ -1509,6 +1625,11 @@ char	*cp;
 	if (current_not_defined()) {
 		return false;
 	}
+	if (strcmp_tospace(cp, "repeat") == 0) {
+		repeat_flag = true;
+		cp = skip_param(cp);
+	}
+	cp_start = cp;
 next_var:
 	if (vc) {
 		v = va[--vc];
@@ -1552,6 +1673,13 @@ next_var:
 			return false;
 		}
 	} else {
+		if (repeat_flag) {
+			if (success_flag) {
+				success_flag = false;
+				cp = cp_start;
+				goto next_var;	/* repeat until failure to substitute anything */
+			}
+		}
 		if (did_something) {
 			did_something = return_result(cur_equation);
 		} else {
@@ -1561,13 +1689,14 @@ next_var:
 	}
 	if (!var_in_equation(cur_equation, v)) {
 #if	!SILENT
-		list_var(v, 0);
-		printf(_("Variable (%s) not found in current equation.\n"), var_str);
+		if (!repeat_flag) {
+			list_var(v, 0);
+			printf(_("Variable (%s) not found in current equation.\n"), var_str);
+		}
 #endif
 		goto next_var;
 	}
-	using_flag = (strcmp_tospace(cp, "using") == 0);
-	if (using_flag) {
+	if (strcmp_tospace(cp, "using") == 0) {
 		cp = skip_param(cp);
 		i = decstrtol(cp, &cp) - 1;
 		if (not_defined(i)) {
@@ -1598,6 +1727,7 @@ next_var:
 			}
 		}
 	}
+	success_flag = true;
 	did_something = true;
 	used[i] = true;
 	goto next_var;
@@ -1661,7 +1791,7 @@ char	*cp;
 				return(result_str != NULL);
 			}
 #endif
-			flist_sub(i);
+			flist_equation(i);
 		}
 	}
 	return true;
@@ -1675,32 +1805,36 @@ list_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	export_flag = false;
+	char	*cp1;
+	int	export_flag = 0;
 
-	if (strcmp_tospace(cp, "export") == 0) {
+	if (strncasecmp(cp, "export", 4) == 0) {
 		export_flag = 2;
 		cp = skip_param(cp);
-	}
-	if (strcmp_tospace(cp, "maxima") == 0) {
+	} else if (strncasecmp(cp, "maxima", 4) == 0) {
 		export_flag = 1;
 		cp = skip_param(cp);
 	}
-	if (!get_range_eol(&cp, &i, &j)) {
-		return false;
-	}
-	for (; i <= j; i++) {
-		if (factor_int_flag) {
-			factor_int_sub(i);
+	do {
+		cp1 = cp;
+		if (!get_range(&cp, &i, &j)) {
+			return false;
 		}
+		if (*cp && cp == cp1) {
+			error(_("Invalid argument."));
+			return false;
+		}
+		for (; i <= j; i++) {
 #if	LIBRARY
-		if (gfp == stdout) {
-			free(result_str);
-			result_str = list_equation(i, export_flag);
-			return(result_str != NULL);
-		}
+			if (gfp == stdout) {
+				free(result_str);
+				result_str = list_equation(i, export_flag);
+				return(result_str != NULL);
+			}
 #endif
-		list1_sub(i, export_flag);
-	}
+			list1_sub(i, export_flag);
+		}
+	} while (*cp);
 	return true;
 }
 
@@ -1715,6 +1849,7 @@ char	*cp;
 	int	i, j;
 	int	language = 0;
 	int	int_flag = false, displayed = false;
+	char	*cp1;
 
 	if (strcmp_tospace(cp, "c") == 0 || strcmp_tospace(cp, "c++") == 0) {
 		cp = skip_param(cp);
@@ -1728,36 +1863,168 @@ char	*cp;
 		cp = skip_param(cp);
 		int_flag = true;
 	}
-	if (!get_range_eol(&cp, &i, &j)) {
-		return false;
-	}
-	for (; i <= j; i++) {
-		if (n_lhs[i] > 0) {
-			if (n_rhs[i] == 0 || n_lhs[i] != 1 || lhs[i][0].kind != VARIABLE) {
-				error(_("Not a solved equation."));
-				continue;
-			}
-			approximate_roots = true;
-			simp_i(rhs[i], &n_rhs[i]);
-			approximate_roots = false;
-			if (int_flag) {
-				uf_repeat_always(rhs[i], &n_rhs[i]);
-			}
-			make_fractions_and_group(i);
-			if (int_flag) {
-				if (!int_expr(rhs[i], n_rhs[i])) {
+	do {
+		cp1 = cp;
+		if (!get_range(&cp, &i, &j)) {
+			return false;
+		}
+		if (*cp && cp == cp1) {
+			error(_("Invalid argument."));
+			return false;
+		}
+		for (; i <= j; i++) {
+			if (n_lhs[i] > 0) {
+				if (n_rhs[i] == 0 || n_lhs[i] != 1 || lhs[i][0].kind != VARIABLE) {
+					error(_("Not a solved equation."));
+					continue;
+				}
+				approximate_roots = true;
+				simp_i(rhs[i], &n_rhs[i]);
+				approximate_roots = false;
+				if (int_flag) {
+					uf_repeat_always(rhs[i], &n_rhs[i]);
+				}
+				make_fractions_and_group(i);
+				if (int_flag && !int_expr(rhs[i], n_rhs[i])) {
 					error(_("Not an integer equation."));
 					continue;
 				}
+				list_c_equation(i, language, int_flag);
+				displayed = true;
 			}
-			list_c_equation(i, language, int_flag);
-			displayed = true;
 		}
-	}
+	} while (*cp);
 	return displayed;
 }
+
+/*
+ * The variables command.
+ */
+int
+variables_cmd(cp)
+char	*cp;
+{
+	int		i, j, k;
+	int		i1;
+	int		vc, cnt;
+	long		v1, last_v;
+	sort_type	va[MAX_VARS];
+	token_type	*p1;
+	int		n1;
+	int		lang_code = 0;
+	int		int_flag = false;
+
+	if (strcmp_tospace(cp, "c") == 0 || strcmp_tospace(cp, "c++") == 0) {
+		cp = skip_param(cp);
+		lang_code = 1;
+	} else if (strcmp_tospace(cp, "java") == 0) {
+		cp = skip_param(cp);
+		lang_code = 2;
+	} else if (strcmp_tospace(cp, "int") == 0 || strcmp_tospace(cp, "integer") == 0) {
+		cp = skip_param(cp);
+		lang_code = 1;
+		int_flag = true;
+	}
+	if (!get_range_eol(&cp, &i, &j)) {
+		return false;
+	}
+	last_v = 0;
+	for (vc = 0;; vc++) {
+		if (vc >= ARR_CNT(va)) {
+			error(_("Too many variables to list."));
+			return false;
+		}
+		cnt = 0;
+		v1 = -1;
+		for (k = i; k <= j; k++) {
+			if (n_lhs[k] <= 0)
+				continue;
+			p1 = lhs[k];
+			n1 = n_lhs[k];
+			for (i1 = 0; i1 < n1; i1 += 2) {
+				if (p1[i1].kind == VARIABLE && p1[i1].token.variable > last_v) {
+					if (v1 == -1 || p1[i1].token.variable < v1) {
+						v1 = p1[i1].token.variable;
+						cnt = 1;
+					} else if (p1[i1].token.variable == v1) {
+						cnt++;
+					}
+				}
+			}
+			p1 = rhs[k];
+			n1 = n_rhs[k];
+			for (i1 = 0; i1 < n1; i1 += 2) {
+				if (p1[i1].kind == VARIABLE && p1[i1].token.variable > last_v) {
+					if (v1 == -1 || p1[i1].token.variable < v1) {
+						v1 = p1[i1].token.variable;
+						cnt = 1;
+					} else if (p1[i1].token.variable == v1) {
+						cnt++;
+					}
+				}
+			}
+		}
+		if (v1 == -1)
+			break;
+		last_v = v1;
+		va[vc].v = v1;
+		va[vc].count = cnt;
+	}
+	if (vc <= 0)
+		return true;
+	qsort((char *) va, vc, sizeof(*va), vcmp);
+	for (i1 = 0; i1 < vc; i1++) {
+		if (lang_code && va[i1].v < SIGN) {
+			continue;
+		}
+		list_var(va[i1].v, lang_code);
+		if (lang_code) {
+			if (int_flag) {
+				fprintf(gfp, "int\t%s;\n", var_str);
+			} else {
+				fprintf(gfp, "double\t%s;\n", var_str);
+			}
+		} else {
+			fprintf(gfp, "%s\n", var_str);
+		}
+	}
+	return true;
+}
 #endif
 
+/*
+ * The approximate command.
+ */
+int
+approximate_cmd(cp)
+char	*cp;
+{
+	int	i, j;
+	char	*cp1;
+
+	do {
+		cp1 = cp;
+		if (!get_range(&cp, &i, &j)) {
+			return false;
+		}
+		if (*cp && cp == cp1) {
+			error(_("Invalid argument."));
+			return false;
+		}
+		for (; i <= j; i++) {
+			if (n_lhs[i]) {
+				subst_constants(lhs[i], &n_lhs[i]);
+				subst_constants(rhs[i], &n_rhs[i]);
+				approximate_roots = true;
+				simp_sub(i);
+				approximate_roots = false;
+				return_result(i);
+			}
+		}
+	} while (*cp);
+	return true;
+}
+
 /*
  * The replace command.
  */
@@ -1779,17 +2046,6 @@ char	*cp;
 		return false;
 	}
 	i = cur_equation;
-	if (strcmp_tospace(cp, "constants") == 0) {
-		cp = skip_param(cp);
-		if (extra_characters(cp))
-			return false;
-		subst_constants(lhs[i], &n_lhs[i]);
-		subst_constants(rhs[i], &n_rhs[i]);
-		approximate_roots = true;
-		simp_sub(i);
-		approximate_roots = false;
-		return return_result(i);
-	}
 	for (vc = 0; *cp; vc++) {
 		if (strcmp_tospace(cp, "with") == 0) {
 			if (vc)
@@ -1898,7 +2154,7 @@ simplify_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	quick_flag = false, symb = false, frac = false;
+	int	quick_flag = false, symb = false, frac_flag = false;
 
 	for (;; cp = skip_param(cp)) {
 		if (strncasecmp(cp, "symbolic", 4) == 0) {
@@ -1910,7 +2166,7 @@ char	*cp;
 			continue;
 		}
 		if (strncasecmp(cp, "fraction", 4) == 0) {
-			frac = true;
+			frac_flag = true;
 			continue;
 		}
 		break;
@@ -1921,8 +2177,8 @@ char	*cp;
 	symb_flag = symb;
 	for (; i <= j; i++) {
 		if (n_lhs[i]) {
-			simpa_side(lhs[i], &n_lhs[i], quick_flag, frac);
-			simpa_side(rhs[i], &n_rhs[i], quick_flag, frac);
+			simpa_side(lhs[i], &n_lhs[i], quick_flag, frac_flag);
+			simpa_side(rhs[i], &n_rhs[i], quick_flag, frac_flag);
 			return_result(i);
 		}
 	}
@@ -1941,31 +2197,46 @@ char	*cp;
 	int	i1, count;
 	long	v;
 	double	d;
-	char	buf[MAX_CMD_LEN];
+	char	*cp_start, buf[MAX_CMD_LEN];
+	int	valid_range = false;
+	int	repeat_flag = false;
 
-	if (strncasecmp(cp, "number", 6) == 0) {
+	cp_start = cp;
+	if (strcmp_tospace(cp, "number") == 0) {
 		cp = skip_param(cp);
-		if (*cp == '\0') {
-			my_strlcpy(prompt_str, _("Enter integers to factor: "), sizeof(prompt_str));
-			cp = get_string(buf, sizeof(buf));
-			if (cp == NULL)
-				return false;
+	} else if (strcmp_tospace(cp, "numbers") == 0) {
+		repeat_flag = true;
+		cp = skip_param(cp);
+	} else {
+		valid_range = get_range(&cp, &i, &j);
+		if (!valid_range) {
+			cp = cp_start;
 		}
-		for (count = 1; *cp; count++) {
-			d = strtod(cp, &cp);
-			cp = skip_space(cp);
-			if (!factor_one(d)) {
-				error(_("Not a valid integer or number too large."));
-				return false;
+	}
+	if (!valid_range) {
+		do {
+			if (*cp == '\0') {
+				my_strlcpy(prompt_str, _("Enter integers to factor: "), sizeof(prompt_str));
+				cp = get_string(buf, sizeof(buf));
+				if (cp == NULL)
+					return false;
 			}
-			display_unique();
-		}
+			if (*cp == '\0') {
+				return true;
+			}
+			for (count = 1; *cp; count++) {
+				d = strtod(cp, &cp);
+				cp = skip_space(cp);
+				if (!factor_one(d)) {
+					error(_("Not a valid integer or number too large."));
+					return false;
+				}
+				display_unique();
+			}
+		} while (repeat_flag);
 		return true;
 	}
 	v = 0;
-	if (!get_range(&cp, &i, &j)) {
-		return false;
-	}
 	do {
 		if (*cp) {
 			if ((cp = parse_var2(&v, cp)) == NULL) {
@@ -2103,7 +2374,7 @@ FILE	*fp;
 	char	*cp;
 
 	blt(save_save, jmp_save, sizeof(jmp_save));
-	if ((rv = setjmp(jmp_save)) != 0) {
+	if ((rv = setjmp(jmp_save)) != 0) {	/* trap errors */
 		clean_up();
 		if (rv == 14) {
 			error(_("Expression too large."));
@@ -2113,6 +2384,9 @@ FILE	*fp;
 			default_color();
 			input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
 			printf("%s", cp);
+			if (gfp != stdout) {
+				fprintf(gfp, "%d%s%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT, cp);
+			}
 			set_error_level(cp);
 			if (!process(cp)) {
 				longjmp(jmp_save, 3);
@@ -2139,7 +2413,7 @@ char	*cp;
 
 	if (*cp == '\0') {
 #if	CYGWIN
-		my_strlcpy(tmp_file, "math.tmp", sizeof(tmp_file));
+		my_strlcpy(tmp_file, "mathomatic.tmp", sizeof(tmp_file));
 		fp = fopen(tmp_file, "w+");
 		if (fp == NULL) {
 			error(_("Can't create temporary file."));
@@ -2177,23 +2451,21 @@ char	*cp;
 {
 	char	cl[MAX_CMD_LEN];	/* command line */
 	char	*cp1;
-	char	*editor_keyword = "EDITOR";
 
 edit_again:
-	cp1 = getenv(editor_keyword);
+	cp1 = getenv("EDITOR");
 	if (cp1 == NULL) {
 #if	CYGWIN
 		cp1 = "notepad";
 #else
-		printf(_("%s environment variable not set.\n"), editor_keyword);
+		error("EDITOR environment variable not set.");
 		return false;
 #endif
 	}
 	snprintf(cl, sizeof(cl), "%s %s", cp1, cp);
 	if (shell_out(cl)) {
-		printf(_("Error executing editor.\n"));
+		error("Error executing editor, check EDITOR environment variable.");
 		printf(_("Command line = \"%s\".\n"), cl);
-		printf(_("Check %s environment variable.\n"), editor_keyword);
 		return false;
 	}
 	clear_all();
@@ -2223,7 +2495,7 @@ char	*cp;
 	}
 #if	!SILENT
 	if (access(cp, 0) == 0) {
-		snprintf(prompt_str, sizeof(prompt_str), _("\"%s\" exists.  Overwrite (Y/N)? "), cp);
+		snprintf(prompt_str, sizeof(prompt_str), _("\"%s\" exists, overwrite (y/n)? "), cp);
 		if (!get_yes_no()) {
 			printf(_("Command aborted.\n"));
 			return false;
@@ -2232,7 +2504,7 @@ char	*cp;
 #endif
 	fp = fopen(cp, "w");
 	if (fp == NULL) {
-		error(_("Can't create file."));
+		error(_("Can't create specified file."));
 		return false;
 	}
 	gfp = fp;
