@@ -1,7 +1,7 @@
 /*
  * Mathomatic differentiation routines and commands.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -314,7 +314,7 @@ char	*cp;
 			cp = skip_param(cp);
 			v = MATCH_ANY;
 		} else {
-			if (!isdigit(*cp)) {
+			if (isvarchar(*cp)) {
 				cp = parse_var2(&v, cp);
 				if (cp == NULL) {
 					return false;
@@ -344,11 +344,10 @@ char	*cp;
 	}
 #if	!SILENT
 	list_var(v, 0);
-	printf(_("Differentiating with respect to (%s) and simplifying"), var_str);
 	if (n_rhs[cur_equation]) {
-		printf(_(" the RHS...\n"));
+		printf(_("Differentiating the RHS with respect to (%s) and simplifying...\n"), var_str);
 	} else {
-		printf("...\n");
+		printf(_("Differentiating with respect to (%s) and simplifying...\n"), var_str);
 	}
 #endif
 	blt(dest, source, *nps * sizeof(token_type));
@@ -400,7 +399,7 @@ char	*cp;
 		n = n_lhs[cur_equation];
 	}
 	if (*cp) {
-		if (!isdigit(*cp)) {
+		if (isvarchar(*cp)) {
 			cp = parse_var2(&v, cp);
 			if (cp == NULL) {
 				return false;
@@ -474,6 +473,7 @@ char	*cp;
 	int		our_nlhs, our_nrhs;
 	token_type	*ep, *source, *dest;
 	int		n1, *nps, *np;
+	int		flag = false;
 
 	cp_start = cp;
 	if (current_not_defined()) {
@@ -500,7 +500,7 @@ char	*cp;
 		dest = lhs[i];
 		np = &n_lhs[i];
 	}
-	if (*cp && !isdigit(*cp)) {
+	if (*cp && isvarchar(*cp)) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
@@ -546,7 +546,7 @@ char	*cp;
 	} else {
 #if	!SILENT
 		list_var(v, 0);
-		printf(_("Taylor series expansion about %s = point.\n"), var_str);
+		printf(_("Taylor series expansion around %s = point.\n"), var_str);
 #endif
 		my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
 		if (!get_expr(lhs[our], &our_nlhs)) {
@@ -631,8 +631,12 @@ loop_again:
 	n1 += 4;
 	for (; i1 < n1; i1++)
 		dest[i1].level++;
-	simp_loop(dest, &n1);
-/*	side_debug(1, dest, n1); */
+	uf_simp(dest, &n1);
+	if (!flag && exp_contains_infinity(dest, n1)) {
+		debug_string(0, _("Singularity warning: result invalid because it contains infinity or NaN."));
+		flag = true;
+	}
+	side_debug(1, dest, n1);
 	if (n < order) {
 		if (n > 0) {
 			if (!differentiate(rhs[our], &our_nrhs, v)) {
@@ -640,7 +644,7 @@ loop_again:
 				return false;
 			}
 		}
-		simpa_side(rhs[our], &our_nrhs, true, false);
+		simpa_side(rhs[our], &our_nrhs, true, true);
 		if (our_nrhs != 1 || rhs[our][0].kind != CONSTANT || rhs[our][0].token.constant != 0.0) {
 			i1 = n1;
 			if ((i1 + 1 + our_nrhs) > n_tokens)
@@ -738,33 +742,31 @@ char	*cp;
 	}
 	if (extra_characters(cp))
 		return false;
-/* copy to a new equation space and work on the copy: */
+/* copy the current equation to a new equation space and work on the copy: */
 	copy_espace(cur_equation, i);
-	cur_equation = i;
 /* see if the limit expression contains infinity: */
 	simp_loop(tes, &n_tes);
 	infinity_flag = exp_contains_infinity(tes, n_tes);
 /* first fully simplify the current equation and group denominators together: */
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
-	group_proc(rhs[cur_equation], &n_rhs[cur_equation]);
+	simpa_side(rhs[i], &n_rhs[i], false, false);
+	group_proc(rhs[i], &n_rhs[i]);
 	found_den = complex_den = false;
-	for (j = 1; j < n_rhs[cur_equation]; j += 2) {
-		switch (rhs[cur_equation][j].token.operatr) {
+	for (j = 1; j < n_rhs[i]; j += 2) {
+		switch (rhs[i][j].token.operatr) {
 		case DIVIDE:
 		case MODULUS:
 			num = den = false;
-			level = rhs[cur_equation][j].level;
+			level = rhs[i][j].level;
 			for (k = j + 2;; k += 2) {
-				if (rhs[cur_equation][k-1].kind == VARIABLE
-				    && rhs[cur_equation][k-1].token.variable == v) {
+				if (rhs[i][k-1].kind == VARIABLE && rhs[i][k-1].token.variable == v) {
 /* the limit variable was found in a denominator */
 					den = true;
 					found_den = true;
 				}
-				if (k >= n_rhs[cur_equation] || rhs[cur_equation][k].level <= level) {
+				if (k >= n_rhs[i] || rhs[i][k].level <= level) {
 					break;
 				}
-				switch (rhs[cur_equation][k].token.operatr) {
+				switch (rhs[i][k].token.operatr) {
 				case DIVIDE:
 				case MODULUS:
 /* a denominator contains a fraction */
@@ -772,9 +774,8 @@ char	*cp;
 					break;
 				}
 			}
-			for (k = j - 1; k >= 0 && rhs[cur_equation][k].level >= level; k--) {
-				if (rhs[cur_equation][k].kind == VARIABLE
-				    && rhs[cur_equation][k].token.variable == v) {
+			for (k = j - 1; k >= 0 && rhs[i][k].level >= level; k--) {
+				if (rhs[i][k].kind == VARIABLE && rhs[i][k].token.variable == v) {
 /* the limit variable was found in a numerator */
 					num = true;
 				}
@@ -788,9 +789,8 @@ char	*cp;
 	if (infinity_flag ? (!complex_den && found_den) : (!found_den)) {
 /* in this case, just substitute the limit variable with the limit expression and simplify: */
 		debug_string(0, "Substituting...");
-		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tes, n_tes, v);
-		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
-		return return_result(cur_equation);
+		subst_var_with_exp(rhs[i], &n_rhs[i], tes, n_tes, v);
+		goto finish_up;
 	}
 	debug_string(0, "Solving...");
 	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == INFINITY) {
@@ -806,31 +806,40 @@ char	*cp;
 		tlhs[2].kind = VARIABLE;
 		tlhs[2].token.variable = v;
 		n_tlhs = 3;
-		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tlhs, n_tlhs, v);
+		subst_var_with_exp(rhs[i], &n_rhs[i], tlhs, n_tlhs, v);
 	}
 /* General limit taking, solve for the limit variable: */
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
-	if (solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) <= 0) {
+	if (solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0) {
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
 /* replace the limit variable (LHS) with the limit expression: */
-	blt(lhs[cur_equation], tes, n_tes * sizeof(token_type));
-	n_lhs[cur_equation] = n_tes;
+	blt(lhs[i], tes, n_tes * sizeof(token_type));
+	n_lhs[i] = n_tes;
 /* symbolically simplify the RHS: */
 	symb_flag = true;
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+	simpa_side(rhs[i], &n_rhs[i], false, false);
 	symb_flag = false;
+	if (exp_contains_nan(rhs[i], n_rhs[i])) {
+		error(_("Unable to take limit; result contains NaN (Not a Number)."));
+		return false;
+	}
 /* solve back for the original variable: */
-	if (solve_sub(&solved_v, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) <= 0) {
+	if (solve_sub(&solved_v, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0) {
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
+finish_up:
 /* symbolically simplify before returning the result: */
 	symb_flag = true;
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+	simpa_side(rhs[i], &n_rhs[i], false, false);
 	symb_flag = false;
-	return return_result(cur_equation);
+	if (exp_contains_nan(rhs[i], n_rhs[i])) {
+		error(_("Unable to take limit; result contains NaN (Not a Number)."));
+		return false;
+	}
+	return return_result(i);
 }
