@@ -1,7 +1,7 @@
 /*
  * Mathomatic global variable extern definitions, from file "globals.c".
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 extern int		n_tokens;
@@ -36,13 +36,14 @@ extern int		display2d;
 extern int		approximate_roots;
 extern int		preserve_roots;
 extern int		true_modulus;
-extern int		screen_columns;
-extern int		screen_rows;
+extern volatile int	screen_columns;
+extern volatile int	screen_rows;
 extern int		finance_option;
 extern int		autosolve;
 extern int		autocalc;
 extern char		special_variable_character;
 extern int		integer_coefficients;
+extern int		right_associative_power;
 extern int		debug_level;
 extern int		domain_check;
 extern int		color_flag;
@@ -71,6 +72,7 @@ extern FILE		*gfp;
 extern jmp_buf		jmp_save;
 extern int		test_mode;
 extern int		quiet_mode;
+extern volatile int	abort_flag;
 
 extern char		*error_str;
 extern char		*result_str;
