@@ -2,12 +2,13 @@
  * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
 
 #define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
+#define MODULUS_STRING	" % "	/* string displayed for the modulus binary operator */
 
 #define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
 
@@ -99,7 +100,7 @@ default_color()
  * Display the expression or equation stored in equation space "n" in single-line format.
  * The equation space is not modified.
  *
- * Return true if successful.
+ * Return length (number of screen columns) of output line.
  */
 int
 list1_sub(n, export_flag)
@@ -107,18 +108,20 @@ int	n;		/* equation space number */
 int	export_flag;	/* true for exportable format (readable by other math programs) */
 			/* 1 for Maxima, 2 for other */
 {
+	int	len = 0;
+
 	if (n_lhs[n] <= 0)
-		return false;
+		return 0;
 	if (!export_flag && !high_prec) {
-		fprintf(gfp, "#%d: ", n + 1);
+		len += fprintf(gfp, "#%d: ", n + 1);
 	}
-	list_proc(lhs[n], n_lhs[n], export_flag);
+	len += list_proc(lhs[n], n_lhs[n], export_flag);
 	if (n_rhs[n]) {
-		fprintf(gfp, EQUATE_STRING);
-		list_proc(rhs[n], n_rhs[n], export_flag);
+		len += fprintf(gfp, EQUATE_STRING);
+		len += list_proc(rhs[n], n_rhs[n], export_flag);
 	}
 	if (export_flag == 1) {
-		fprintf(gfp, ";");
+		len += fprintf(gfp, ";");
 	}
 	if (high_prec) {
 #if	CYGWIN
@@ -129,27 +132,28 @@ int	export_flag;	/* true for exportable format (readable by other math programs)
 	} else {
 		fprintf(gfp, "\n\n");
 	}
-	return true;
+	return len;
 }
 
 /*
  * Display the expression or equation stored in equation space "n".
  *
- * Return true if successful.
+ * Return the total width of the output (number of screen columns)
+ * or zero on failure.
  */
 int
 list_sub(n)
 int	n;	/* equation space number */
 {
 	if (n_lhs[n] <= 0)
-		return false;
+		return 0;
 	if (display2d) {
 		/* display in fraction format */
 		make_fractions_and_group(n);
 		if (factor_int_flag) {
 			factor_int_sub(n);
 		}
-		return flist_sub(n);
+		return flist_equation(n);
 	} else {
 		/* display in single-line format */
 		if (factor_int_flag) {
@@ -160,6 +164,7 @@ int	n;	/* equation space number */
 }
 
 #if	!SILENT
+void
 list_debug(level, p1, n1, p2, n2)
 int		level;
 token_type	*p1;
@@ -180,16 +185,16 @@ int		n2;
 #endif
 
 /*
- * Convert variable "v" to an ASCII variable name.
- * The ASCII variable name is stored in global "var_str[]".
+ * Convert the passed Mathomatic variable to an ASCII variable name.
+ * The ASCII variable name is stored in the global var_str[].
  *
  * Return length of variable name (number of ASCII characters).
  *
- * If "lang_code == 0", use standard Mathomatic format.
- * If "lang_code == 1", make variable name C language compatible.
- * If "lang_code == 2", assume Java.
- * If "lang_code == 3", assume Python.
- * If "lang_code" is negative, create an exportable variable name: -1 for Maxima, -2 for other.
+ * If (lang_code == 0), use standard Mathomatic format.
+ * If (lang_code == 1), make variable name C language compatible.
+ * If (lang_code == 2), make variable name Java compatible.
+ * If (lang_code == 3), make variable name Python compatible.
+ * If lang_code is negative, create an exportable variable name: -1 for Maxima, -2 for other.
  */
 int
 list_var(v, lang_code)
@@ -283,7 +288,7 @@ int	lang_code;	/* language code */
 	if (cp) {
 		my_strlcpy(var_str, cp, sizeof(var_str));
 	} else {
-		my_strlcpy(var_str, _("bad_variable"), sizeof(var_str));
+		my_strlcpy(var_str, "bad_variable", sizeof(var_str));
 	}
 	j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
 	if (j) {
@@ -301,8 +306,8 @@ int	lang_code;	/* language code */
  * Return number of characters output (excluding escape sequences).
  */
 int
-list_proc(equation, n, export_flag)
-token_type	*equation;	/* expression pointer */
+list_proc(p1, n, export_flag)
+token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		export_flag;	/* flag for exportable format (usually false) */
 				/* 1 for Maxima, 2 for other */
@@ -313,9 +318,9 @@ int		export_flag;	/* flag for exportable format (usually false) */
 	char	*cp;
 	int	len = 0;
 
-	cur_level = min1 = min_level(equation, n);
+	cur_level = min1 = min_level(p1, n);
 	for (i = 0; i < n; i++) {
-		j = cur_level - equation[i].level;
+		j = cur_level - p1[i].level;
 		k = abs(j);
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
@@ -328,30 +333,30 @@ int		export_flag;	/* flag for exportable format (usually false) */
 				len += fprintf(gfp, "(");
 			}
 		}
-		switch (equation[i].kind) {
+		switch (p1[i].kind) {
 		case CONSTANT:
-			if (equation[i].token.constant == 0.0) {
-				equation[i].token.constant = 0.0; /* fix -0 */
+			if (p1[i].token.constant == 0.0) {
+				p1[i].token.constant = 0.0; /* fix -0 */
 			}
 			if (high_prec || export_flag) {
-				if (equation[i].token.constant < 0.0) {
-					len += fprintf(gfp, "(%.20g)", equation[i].token.constant);
+				if (p1[i].token.constant < 0.0) {
+					len += fprintf(gfp, "(%.20g)", p1[i].token.constant);
 				} else {
-					len += fprintf(gfp, "%.20g", equation[i].token.constant);
+					len += fprintf(gfp, "%.20g", p1[i].token.constant);
 				}
 			} else if (finance_option) {
-				len += fprintf(gfp, "%.2f", equation[i].token.constant);
+				len += fprintf(gfp, "%.*f", finance_option, p1[i].token.constant);
 			} else {
-				len += fprintf(gfp, "%.*g", precision, equation[i].token.constant);
+				len += fprintf(gfp, "%.*g", precision, p1[i].token.constant);
 			}
 			break;
 		case VARIABLE:
-			list_var(equation[i].token.variable, 0 - export_flag);
+			list_var(p1[i].token.variable, 0 - export_flag);
 			len += fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
 			cp = _("(unknown operator)");
-			switch (equation[i].token.operatr) {
+			switch (p1[i].token.operatr) {
 			case PLUS:
 				cp = " + ";
 				break;
@@ -365,7 +370,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
 				cp = "/";
 				break;
 			case MODULUS:
-				cp = " % ";
+				cp = MODULUS_STRING;
 				break;
 			case POWER:
 				cp = "^";
@@ -391,7 +396,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
  * Store the specified equation in a text string.
  * String should be freed with free() when done.
  *
- * Return string or NULL if can't malloc().
+ * Return string or NULL if error.
  */
 char *
 list_equation(n, export_flag)
@@ -401,6 +406,9 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	int	len;
 	char	*cp;
 
+	if (n_lhs[n] <= 0) {
+		return NULL;
+	}
 	len = list_string_sub(lhs[n], n_lhs[n], NULL, export_flag);
 	if (n_rhs[n]) {
 		len += strlen(EQUATE_STRING);
@@ -427,25 +435,28 @@ int	export_flag;	/* flag for exportable format (usually false) */
  * Store an expression in a text string.
  * String should be freed with free() when done.
  *
- * Return string or NULL if can't malloc().
+ * Return string or NULL if error.
  */
 char *
-list_expression(equation, n, export_flag)
-token_type	*equation;	/* expression pointer */
+list_expression(p1, n, export_flag)
+token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		export_flag;
 {
 	int	len;
 	char	*cp;
 
-	len = list_string_sub(equation, n, NULL, export_flag);
+	if (n <= 0) {
+		return NULL;
+	}
+	len = list_string_sub(p1, n, NULL, export_flag);
 	len++;
 	cp = (char *) malloc(len);
 	if (cp == NULL) {
 		error(_("Out of memory (can't malloc(3))."));
 		return NULL;
 	}
-	list_string_sub(equation, n, cp, export_flag);
+	list_string_sub(p1, n, cp, export_flag);
 	return cp;
 }
 
@@ -457,8 +468,8 @@ int		export_flag;
  * Return length (number of characters).
  */
 int
-list_string_sub(equation, n, string, export_flag)
-token_type	*equation;	/* expression pointer */
+list_string_sub(p1, n, string, export_flag)
+token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 char		*string;	/* buffer or NULL pointer */
 int		export_flag;
@@ -472,9 +483,9 @@ int		export_flag;
 
 	if (string)
 		string[0] = '\0';
-	cur_level = min1 = min_level(equation, n);
+	cur_level = min1 = min_level(p1, n);
 	for (i = 0; i < n; i++) {
-		j = cur_level - equation[i].level;
+		j = cur_level - p1[i].level;
 		k = abs(j);
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
@@ -485,31 +496,31 @@ int		export_flag;
 				APPEND("(");
 			}
 		}
-		switch (equation[i].kind) {
+		switch (p1[i].kind) {
 		case CONSTANT:
-			if (equation[i].token.constant == 0.0) {
-				equation[i].token.constant = 0.0; /* fix -0 */
+			if (p1[i].token.constant == 0.0) {
+				p1[i].token.constant = 0.0; /* fix -0 */
 			}
 			if (high_prec || export_flag) {
-				if (equation[i].token.constant < 0.0) {
-					snprintf(buf, sizeof(buf), "(%.20g)", equation[i].token.constant);
+				if (p1[i].token.constant < 0.0) {
+					snprintf(buf, sizeof(buf), "(%.20g)", p1[i].token.constant);
 				} else {
-					snprintf(buf, sizeof(buf), "%.20g", equation[i].token.constant);
+					snprintf(buf, sizeof(buf), "%.20g", p1[i].token.constant);
 				}
 			} else if (finance_option) {
-				snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
+				snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
 			} else {
-				snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
+				snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
 			}
 			APPEND(buf);
 			break;
 		case VARIABLE:
-			list_var(equation[i].token.variable, 0 - export_flag);
+			list_var(p1[i].token.variable, 0 - export_flag);
 			APPEND(var_str);
 			break;
 		case OPERATOR:
 			cp = _("(unknown operator)");
-			switch (equation[i].token.operatr) {
+			switch (p1[i].token.operatr) {
 			case PLUS:
 				cp = " + ";
 				break;
@@ -523,7 +534,7 @@ int		export_flag;
 				cp = "/";
 				break;
 			case MODULUS:
-				cp = " % ";
+				cp = MODULUS_STRING;
 				break;
 			case POWER:
 				cp = "^";
@@ -548,15 +559,15 @@ int		export_flag;
  * list_code().
  */
 int
-int_expr(equation, n)
-token_type	*equation;	/* expression pointer */
+int_expr(p1, n)
+token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 {
 	int	i;
 
 	for (i = 0; i < n; i++) {
-		if ((equation[i].kind == CONSTANT && fmod(equation[i].token.constant, 1.0))
-		    || (equation[i].kind == VARIABLE && equation[i].token.variable <= IMAGINARY)) {
+		if ((p1[i].kind == CONSTANT && fmod(p1[i].token.constant, 1.0))
+		    || (p1[i].kind == VARIABLE && p1[i].token.variable <= IMAGINARY)) {
 			return false;
 		}
 	}
@@ -593,8 +604,8 @@ int	int_flag;	/* integer arithmetic flag */
  */
 int
 list_code(equation, np, language, int_flag)
-token_type	*equation;	/* expression pointer */
-int		*np;		/* pointer to length of expression */
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
 int		language;	/* if 0, generate C code; if 1, generate Java code; if 2, generate Python code */
 int		int_flag;	/* integer arithmetic flag, should work with any language */
 {
@@ -686,17 +697,17 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 				cp = " - ";
 				break;
 			case TIMES:
-				cp = " * ";
+				cp = "*";
 				break;
 			case DIVIDE:
-				cp = " / ";
+				cp = "/";
 				break;
 			case MODULUS:
-				cp = " % ";
+				cp = MODULUS_STRING;
 				break;
 			case POWER:
 				if (int_flag || language > 1) {
-					cp = " ** ";
+					cp = "**";
 				} else {
 					cp = ", ";
 				}
@@ -720,22 +731,23 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
  * Display an equation space in multi-line fraction format.
  * Use color if available.
  *
- * Return true if successful.
+ * Return the total width of the output (that is, the required number of screen columns)
+ * or zero on failure.
  */
 int
-flist_sub(n)
+flist_equation(n)
 int	n;	/* equation space number */
 {
 	int	sind;
 	char	buf[30];
-	int	len, len2, len3;
+	int	len, len2, len3, width;
 	int	pos;
 	int	high, low;
 	int	max_line, min_line;
 	int	max2_line, min2_line;
 
 	if (n_lhs[n] <= 0)
-		return false;
+		return 0;
 	len = snprintf(buf, sizeof(buf), "#%d: ", n + 1);
 	cur_line = 0;
 	cur_pos = 0;
@@ -767,7 +779,9 @@ make_smaller:
 		len2 = 0;
 		len3 = 0;
 	}
-	if (screen_columns && gfp == stdout && max(len + len2, len3) >= screen_columns) {
+	width = max(len + len2, len3);
+	if (screen_columns && gfp == stdout && width >= screen_columns) {
+		/* output too wide to fit screen, output in single-line format */
 		return list1_sub(n, false);
 	}
 	fprintf(gfp, "\n");
@@ -796,7 +810,7 @@ make_smaller:
 		}
 	}
 	fprintf(gfp, "\n");
-	return true;
+	return width;
 }
 
 /*
@@ -806,8 +820,8 @@ make_smaller:
  * Return the width of the expression (that is, the required number of screen columns).
  */
 static int
-flist(equation, n, out_flag, pos, highp, lowp)
-token_type	*equation;	/* expression pointer */
+flist(p1, n, out_flag, pos, highp, lowp)
+token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		out_flag;
 int		pos;
@@ -815,7 +829,7 @@ int		*highp, *lowp;
 {
 	int	i;
 
-	i = flist_recurse(equation, n, out_flag, 0, pos, 1, highp, lowp);
+	i = flist_recurse(p1, n, out_flag, 0, pos, 1, highp, lowp);
 	if (out_flag) {
 		default_color();
 	}
@@ -823,8 +837,8 @@ int		*highp, *lowp;
 }
 
 static int
-flist_recurse(equation, n, out_flag, line, pos, cur_level, highp, lowp)
-token_type	*equation;
+flist_recurse(p1, n, out_flag, line, pos, cur_level, highp, lowp)
+token_type	*p1;
 int		n;
 int		out_flag;
 int		line;
@@ -864,10 +878,10 @@ check_again:
 	stop_at = n;
 	div_loc = -1;
 	for (i = ii; i < n; i++) {
-		if (equation[i].kind == OPERATOR && equation[i].token.operatr == DIVIDE) {
-			level = equation[i].level;
+		if (p1[i].kind == OPERATOR && p1[i].token.operatr == DIVIDE) {
+			level = p1[i].level;
 			for (j = i - 2; j > 0; j -= 2) {
-				if (equation[j].level < level)
+				if (p1[j].level < level)
 					break;
 			}
 			j++;
@@ -879,7 +893,7 @@ check_again:
 					div_loc = i;
 					stop_at = j;
 				} else if (j == stop_at) {
-					if (level < equation[div_loc].level)
+					if (level < p1[div_loc].level)
 						div_loc = i;
 				}
 			}
@@ -887,10 +901,10 @@ check_again:
 	}
 	for (i = ii; i < n; i++) {
 		if (i == stop_at) {
-			j = cur_level - equation[div_loc].level;
+			j = cur_level - p1[div_loc].level;
 			k = abs(j) - 1;
 		} else {
-			j = cur_level - equation[i].level;
+			j = cur_level - p1[i].level;
 			k = abs(j);
 		}
 		for (i1 = 1; i1 <= k; i1++) {
@@ -911,14 +925,14 @@ check_again:
 			}
 		}
 		if (i == stop_at) {
-			level = equation[div_loc].level;
-			len1 = flist_recurse(&equation[stop_at], div_loc - stop_at, false, line + 1, pos + len, level, &high, &low);
+			level = p1[div_loc].level;
+			len1 = flist_recurse(&p1[stop_at], div_loc - stop_at, false, line + 1, pos + len, level, &high, &low);
 			l1 = (2 * (line + 1)) - low;
 			for (j = div_loc + 2; j < n; j += 2) {
-				if (equation[j].level <= level)
+				if (p1[j].level <= level)
 					break;
 			}
-			len2 = flist_recurse(&equation[div_loc+1], j - (div_loc + 1), false, line - 1, pos + len, level, &high, &low);
+			len2 = flist_recurse(&p1[div_loc+1], j - (div_loc + 1), false, line - 1, pos + len, level, &high, &low);
 			l2 = (2 * (line - 1)) - high;
 			ii = j;
 			len_div = max(len1, len2);
@@ -926,7 +940,7 @@ check_again:
 			if (len1 < len_div) {
 				j = (len_div - len1) / 2;
 			}
-			flist_recurse(&equation[stop_at], div_loc - stop_at, out_flag, l1, pos + len + j, level, &high, &low);
+			flist_recurse(&p1[stop_at], div_loc - stop_at, out_flag, l1, pos + len + j, level, &high, &low);
 			if (high > *highp)
 				*highp = high;
 			if (low < *lowp)
@@ -946,7 +960,7 @@ check_again:
 			if (len2 < len_div) {
 				j = (len_div - len2) / 2;
 			}
-			flist_recurse(&equation[div_loc+1], ii - (div_loc + 1), out_flag, l2, pos + len + j, level, &high, &low);
+			flist_recurse(&p1[div_loc+1], ii - (div_loc + 1), out_flag, l2, pos + len + j, level, &high, &low);
 			if (high > *highp)
 				*highp = high;
 			if (low < *lowp)
@@ -954,65 +968,71 @@ check_again:
 			len += len_div;
 			goto check_again;
 		}
-		switch (equation[i].kind) {
+		switch (p1[i].kind) {
 		case CONSTANT:
-			if (equation[i].token.constant == 0.0) {
-				equation[i].token.constant = 0.0; /* fix -0 */
+			if (p1[i].token.constant == 0.0) {
+				p1[i].token.constant = 0.0; /* fix -0 */
 			}
 			if (finance_option) {
-				len += snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
+				len += snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
 			} else {
-				len += snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
+				len += snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
 			}
 			if (oflag)
 				fprintf(gfp, "%s", buf);
 			break;
 		case VARIABLE:
-			len += list_var(equation[i].token.variable, 0);
+			len += list_var(p1[i].token.variable, 0);
 			if (oflag)
 				fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
-			cp = _("(unknown operator)");
-			switch (equation[i].token.operatr) {
+			/* len must be incremented here by the number of columns the display of the operator takes */
+			switch (p1[i].token.operatr) {
 			case PLUS:
 				cp = " + ";
+				len += 3;
 				break;
 			case MINUS:
-				cp = " - ";
+				if (html_flag) {
+					cp = " &minus; ";
+				} else {
+					cp = " - ";
+				}
+				len += 3;
 				break;
 			case TIMES:
-#if	false	/* if true use HTML times character, instead of "*" */
 				if (html_flag) {
-					cp = "&times;";
+					cp = "&middot;";
 				} else {
 					cp = "*";
 				}
-#else
-				cp = "*";
-#endif
+				len++;
 				break;
 			case DIVIDE:
 				cp = "/";
+				len++;
 				break;
 			case MODULUS:
-				cp = " % ";
+				cp = MODULUS_STRING;
+				len += strlen(cp);
 				break;
 			case POWER:
 				cp = "^";
+				len++;
 				break;
 			case FACTORIAL:
 				cp = "!";
+				len++;
 				i++;
 				break;
+			default:
+				cp = _("(unknown operator)");
+				len += strlen(cp);
+				break;
 			}
 			if (oflag)
 				fprintf(gfp, "%s", cp);
-			if (html_flag && cp[0] == '&') {
-				len++;
-			} else {
-				len += strlen(cp);
-			}
 			break;
 		}
 	}
