@@ -2,7 +2,7 @@
  * This file contains main() and the startup code for Mathomatic.
  * Leave this file out when compiling for the symbolic math library.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  *
  * Output to stderr is only done in this file.  The rest of the code
  * should not output to stderr.
@@ -46,11 +46,11 @@ main(argc, argv)
 int	argc;
 char	**argv;
 {
-	extern char	*optarg;
+	extern char	*optarg;	/* set by getopt(3) */
 	extern int	optind;
 
 	int		i;
-	char		*cp;
+	char		*cp = NULL;
 	double		numerator, denominator;
 	double		multiplier;
 	int		coption = false;
@@ -77,9 +77,9 @@ char	**argv;
 			break;
 		case 'm':
 			multiplier = strtod(optarg, &cp) * DEFAULT_N_TOKENS;
-			if (*cp || multiplier < 100 || multiplier >= (INT_MAX / 3)) {
+			if (cp == NULL || *cp || multiplier < 100 || multiplier >= (INT_MAX / 3)) {
 				fprintf(stderr, _("%s: Invalid memory size multiplier specified.\n"), prog_name);
-				exit(1);
+				exit(2);
 			}
 			n_tokens = (int) multiplier;
 			break;
@@ -121,7 +121,7 @@ char	**argv;
 		printf(_("Secure "));
 #endif
 		printf(_("Mathomatic version %s (www.mathomatic.org)\n"), VERSION);
-		printf(_("Copyright (C) 1987-2007 George Gesslein II.\n"));
+		printf(_("Copyright (C) 1987-2008 George Gesslein II.\n"));
 		printf(_("This is free software; see the source for copying conditions.  There is NO\n"));
 		printf(_("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n"));
 		printf(_("%d equation spaces available, %lld kilobytes per equation space.\n"),
@@ -130,7 +130,7 @@ char	**argv;
 #if	!SECURE
 	/* read the user options initialization file */
 	if (!test_mode && !load_rc()) {
-		fprintf(stderr, _("Error loading set options from \"%s\".\n"), RC_FILE);
+		fprintf(stderr, _("%s: Error loading set options from \"%s\".\n"), prog_name, RC_FILE);
 	}
 #endif
 	if (test_mode) {
@@ -146,21 +146,20 @@ char	**argv;
 		/* for error handling */
 		clean_up();
 		switch (i) {
-		case 13:
-			printf(_("Operation abruptly aborted.\n"));
-			break;
 		case 14:
 			error(_("Expression too large."));
 		default:
 			printf(_("Operation aborted.\n"));
+			break;
 		}
 	} else {
-		if (!set_signals()) {
-			fprintf(stderr, _("signal(2) setting failed.\n"));
-		}
+		set_signals();
 		if (!f_to_fraction(0.5, &numerator, &denominator)
-		    || !f_to_fraction(1.0/3.0, &numerator, &denominator)) {
-			fprintf(stderr, _("Cannot convert floating point values to fractions.\n"));
+		    || numerator != 1.0 || denominator != 2.0
+		    || !f_to_fraction(1.0/3.0, &numerator, &denominator)
+		    || numerator != 1.0 || denominator != 3.0) {
+			fprintf(stderr, _("%s: Cannot convert any floating point values to fractions.\n"), prog_name);
+			fprintf(stderr, _("Roots will not work properly.\n"));
 		}
 #if	!SECURE
 		/* read in files on the command line, exit if error */
@@ -180,6 +179,7 @@ char	**argv;
 		process(cp);
 	}
 	exit_program(0);
+	return(0);	/* so the compiler doesn't complain */
 }
 
 /*
@@ -188,18 +188,18 @@ char	**argv;
 static void
 usage()
 {
-	fprintf(stderr, _("Mathomatic version %s - automatic algebraic manipulator\n\n"), VERSION);
-	fprintf(stderr, _("Usage: %s [ options ] [ input_files ]\n\n"), prog_name);
-	fprintf(stderr, _("Options:\n"));
-	fprintf(stderr, _("  -c               Toggle color mode.\n"));
-	fprintf(stderr, _("  -h               Enable HTML output mode and disable readline.\n"));
-	fprintf(stderr, _("  -m number        Specify a memory size multiplier.\n"));
-	fprintf(stderr, _("  -q               Set quiet mode (don't display prompts).\n"));
-	fprintf(stderr, _("  -r               Disable readline.\n"));
-	fprintf(stderr, _("  -t               Set test mode.\n"));
-	fprintf(stderr, _("  -u               Set unbuffered output.\n"));
-	fprintf(stderr, _("  -v               Display version number, compile flags, and memory usage.\n"));
-	exit(1);
+	fprintf(stderr, "Mathomatic version %s - automatic algebraic manipulator\n\n", VERSION);
+	fprintf(stderr, "Usage: %s [ options ] [ input_files ]\n\n", prog_name);
+	fprintf(stderr, "Options:\n");
+	fprintf(stderr, "  -c            Toggle color mode.\n");
+	fprintf(stderr, "  -h            Enable HTML output mode and disable readline.\n");
+	fprintf(stderr, "  -m number     Specify a memory size multiplier.\n");
+	fprintf(stderr, "  -q            Set quiet mode (don't display prompts).\n");
+	fprintf(stderr, "  -r            Disable readline.\n");
+	fprintf(stderr, "  -t            Set test mode.\n");
+	fprintf(stderr, "  -u            Set unbuffered output.\n");
+	fprintf(stderr, "  -v            Display version number, compile flags, and memory usage.\n");
+	exit(2);
 }
 
 /*
@@ -223,6 +223,9 @@ set_signals()
 		rv = false;
 	alarm(TIMEOUT_SECONDS);
 #endif
+	if (!rv) {
+		perror("signal(2) setting failed");
+	}
 	return rv;
 }
 
@@ -293,19 +296,23 @@ void
 fphandler(sig)
 int	sig;
 {
-	error(_("Floating point exception."));
+/*	error(_("Floating point exception.")); */
 }
 
 /*
  * Control-C (interrupt) handler.
- * Abruptly quit this program.
  */
 void
 inthandler(sig)
 int	sig;
 {
+#if	true
+	abort_flag = true;	/* interrupt processing and return to main prompt */
+#else
+	/* Abruptly quit this program. */
 	printf(_("\nControl-C pressed, quitting...\n"));
 	exit_program(1);
+#endif
 }
 
 /*
@@ -357,7 +364,6 @@ exit_program(exit_value)
 int	exit_value;	/* zero if OK, non-zero indicates error return */
 {
 	reset_attr();
-	printf("\n");
 	if (html_flag) {
 		printf("</pre>\n");
 	}
