@@ -1,7 +1,7 @@
 /*
  * Expression parsing routines for Mathomatic.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -17,7 +17,7 @@ void
 str_tolower(cp)
 char	*cp;
 {
-	for (; *cp; cp++) {
+	for (; cp && *cp; cp++) {
 		if (isupper(*cp))
 			*cp = tolower(*cp);
 	}
@@ -58,27 +58,31 @@ int	ch;
  * Parenthesize an operator.
  */
 void
-binary_parenthesize(equation, n, i)
-token_type	*equation;	/* pointer to expression */
-int		n;		/* length of expression */
-int		i;		/* location of operator to parenthesize in expression */
+binary_parenthesize(p1, n, i)
+token_type	*p1;	/* pointer to expression */
+int		n;	/* length of expression */
+int		i;	/* location of operator to parenthesize in expression */
 {
 	int	j;
 	int	level;
 
-	level = equation[i].level++;
-	if (equation[i-1].level++ > level) {
+	if (i >= (n - 1) || (n & 1) != 1 || (i & 1) != 1) {
+		error("Internal error in arguments to binary_parenthesize().");
+		longjmp(jmp_save, 13);
+	}
+	level = p1[i].level++;
+	if (p1[i-1].level++ > level) {
 		for (j = i - 2; j >= 0; j--) {
-			if (equation[j].level <= level)
+			if (p1[j].level <= level)
 				break;
-			equation[j].level++;
+			p1[j].level++;
 		}
 	}
-	if (equation[i+1].level++ > level) {
+	if (p1[i+1].level++ > level) {
 		for (j = i + 2; j < n; j++) {
-			if (equation[j].level <= level)
+			if (p1[j].level <= level)
 				break;
-			equation[j].level++;
+			p1[j].level++;
 		}
 	}
 }
@@ -121,14 +125,17 @@ int		*np;		/* pointer to expression length */
 		}
 	}
 
-/* To make the power operator evaluate right to left, set the following to false: */
-#if	true
-	for (i = 1; i < *np; i += 2) {		/* count up (left to right) */
-#else
-	for (i = *np - 2; i >= 1; i -= 2) {	/* count down (right to left) */
-#endif
-		if (equation[i].token.operatr == POWER) {
-			binary_parenthesize(equation, *np, i);
+	if (right_associative_power) {
+		for (i = *np - 2; i >= 1; i -= 2) {	/* count down (right to left) */
+			if (equation[i].token.operatr == POWER) {
+				binary_parenthesize(equation, *np, i);
+			}
+		}
+	} else {
+		for (i = 1; i < *np; i += 2) {		/* count up (left to right) */
+			if (equation[i].token.operatr == POWER) {
+				binary_parenthesize(equation, *np, i);
+			}
 		}
 	}
 
@@ -171,9 +178,10 @@ char	*cp;	/* pointer to the beginning of the equation character string */
  * This is a simple, non-recursive mathematical expression parser.
  * To parse, the character string is sequentially read and stored
  * in the internal storage format.
- * Any syntax errors are carefully reported with a NULL return.
+ * The maximum length that can be parsed is the size of an equation side.
+ * Any syntax and other errors are carefully reported and give a NULL return.
  *
- * Returns the new string position or NULL on error.
+ * Returns the new string position or NULL if error.
  */
 char	*
 parse_section(equation, np, cp)
@@ -211,7 +219,7 @@ char		*cp;		/* string to parse */
 		case '}':
 			cur_level--;
 			if (cur_level <= 0 || (abs_count > 0 && cur_level < abs_array[abs_count-1])) {
-				put_up_arrow((int) (cp - cp_start), _("Too many right parentheses."));
+				put_up_arrow((int) (cp - cp_start), _("Unmatched parentheses: too many )."));
 				return(NULL);
 			}
 			if (!operand) {
@@ -235,7 +243,7 @@ char		*cp;		/* string to parse */
 					error(_("Too many nested absolute values."));
 					return(NULL);
 				}
-				cur_level += 2;
+				cur_level += 3;
 				abs_array[abs_count++] = cur_level;
 			} else {
 				if (abs_count <= 0 || cur_level != abs_array[--abs_count]) {
@@ -250,6 +258,7 @@ char		*cp;		/* string to parse */
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = 2.0;
 				n++;
+				cur_level--;
 				equation[n].level = cur_level;
 				equation[n].kind = OPERATOR;
 				equation[n].token.operatr = POWER;
@@ -267,8 +276,7 @@ char		*cp;		/* string to parse */
 				goto syntax_error;
 			}
 			if (cp[1] == '!') {
-				put_up_arrow((int) (cp - cp_start), _("Multifactorial not supported, use (x!)! for the factorial of x factorial."));
-				return(NULL);
+				debug_string(0, _("Multifactorial not implemented, use \"(x!)!\" for the factorial of x factorial."));
 			}
 			equation[n].level = cur_level;
 			equation[n].kind = OPERATOR;
@@ -428,7 +436,7 @@ p_out:
 		goto syntax_error;
 	}
 	if (cur_level != 1) {
-		put_up_arrow((int) (cp - cp_start), _("Missing right parenthesis."));
+		put_up_arrow((int) (cp - cp_start), _("Unmatched parentheses: missing )."));
 		return(NULL);
 	}
 	if (*cp == '=')
@@ -611,7 +619,7 @@ char	*cp;	/* input string */
 }
 
 /*
- * Return true if passed variable is a constant.
+ * Return true if passed variable "v" is a constant.
  * Return value of constant in "*dp".
  */
 int
