@@ -1,19 +1,53 @@
 /*
  * Mathomatic simplifying routines.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
 
-#define	MAX_COMPARE_TERMS	500	/* Max no. of terms on same level that can be compared, increases stack usage. */
-					/* MAX_COMPARE_TERMS should be approximately (sqrt(DEFAULT_N_TOKENS) * 3) */
+/*
+ * The following defines the maximum number of terms on the same level that can be compared.
+ * Most of the stack usage is related this value,
+ * this many pointers are pushed onto the stack with each level of parentheses while comparing two expressions.
+ */
+#define	MAX_COMPARE_TERMS	(DEFAULT_N_TOKENS / 10)
 
 static int	org_recurse();
 static int	const_recurse();
 static int	compare_recurse();
 static int	order_recurse();
 
+/*
+ * The most basic simplification loop.
+ * Just does constant simplification.
+ */
+void
+elim_loop(equation, np)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
+{
+	if (abort_flag) {
+		/* Control-C pressed, return to main prompt and leave unsimplified */
+		abort_flag = false;
+		longjmp(jmp_save, 13);
+	}
+	side_debug(6, equation, *np);
+	do {
+		do {
+			do {
+				organize(equation, np);
+			} while (combine_constants(equation, np, true));
+		} while (elim_k(equation, np));
+	} while (simp_pp(equation, np));
+	if (reorder(equation, np)) {
+		do {
+			organize(equation, np);
+		} while (elim_k(equation, np));
+	}
+	side_debug(5, equation, *np);
+}
+
 /*
  * Quickly and very basically simplify an equation space.
  * No factoring is done.
@@ -48,8 +82,10 @@ void
 simps_side(equation, np, zsolve)
 token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of equation side */
-int		zsolve;		/* true if we are solving for zero */
+int		zsolve;		/* true for solving for zero */
 {
+	elim_loop(equation, np);
+	simp_constant_power(equation, np);
 	do {
 		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 2);
 	} while (super_factor(equation, np, 0));
@@ -62,7 +98,7 @@ void
 simpv_side(equation, np, v)
 token_type	*equation;	/* pointer to the beginning of equation side to factor */
 int		*np;		/* pointer to length of equation side */
-long		v;		/* variable to factor */
+long		v;		/* variable to factor, 0 for all variables */
 {
 	if (*np == 0)
 		return;
@@ -96,6 +132,7 @@ int		*np;
 	approximate_roots = true;
 	subst_constants(equation, np);
 	simp_side(equation, np);
+	uf_power(equation, np);
 	factorv(equation, np, IMAGINARY);
 	ufactor(equation, np);
 	factorv(equation, np, IMAGINARY);
@@ -107,7 +144,8 @@ int		*np;
 }
 
 /*
- * Try to eliminate "i#" from an equation side by converting "i#*(b^.5)" to "(-1*b)^.5".
+ * Try to eliminate imaginary numbers (i#) from an equation side by converting
+ * expressions like (i#*(b^.5)) to ((-1*b)^.5).
  */
 void
 simp_i(equation, np)
@@ -167,12 +205,12 @@ int		*np;		/* pointer to length of the equation side */
 int		quick_flag;	/* "simplify quick" option */
 int		frac_flag;	/* if true, simplify to the ratio of two polynomials, like Maxima's ratsimp() */
 {
+	int		i;
 	int		flag, poly_flag = true;
 	jmp_buf		save_save;
 
 	if (*np <= 1)	/* no need to simplify a single constant or variable */
 		return;
-	partial_flag = 2;
 	simp_loop(equation, np);
 	poly_factor(equation, np);
 	simpb_side(equation, np, true, 1);
@@ -181,11 +219,12 @@ int		frac_flag;	/* if true, simplify to the ratio of two polynomials, like Maxim
 	unsimp_power(equation, np);
 	uf_times(equation, np);
 	simp_loop(equation, np);
+	/* Here is the only place in Mathomatic that we do modulus (%) simplification: */
 	uf_repeat(equation, np);
 	do {
 		elim_loop(equation, np);
 	} while (mod_simp(equation, np));
-	/* Here we try to simplify out unnecessary negative constants and imaginary numbers */
+	/* Here we try to simplify out unnecessary negative constants and imaginary numbers: */
 	simp_i(equation, np);
 	unsimp_power(equation, np);
 	uf_times(equation, np);
@@ -221,12 +260,15 @@ do_quick:
 		n_tlhs = *np;
 		blt(tlhs, equation, n_tlhs * sizeof(token_type));
 		blt(save_save, jmp_save, sizeof(jmp_save));
-		if (setjmp(jmp_save) != 0) {
+		if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
 			blt(jmp_save, save_save, sizeof(jmp_save));
-			/* restore the original expression */
+			if (i == 13) {
+				longjmp(jmp_save, 13);
+			}
+			/* an error occurred, restore the original expression */
 			*np = n_tlhs;
 			blt(equation, tlhs, n_tlhs * sizeof(token_type));
-			goto do_quick;
+			goto do_quick;	/* skip power expansion */
 		}
 		/* expand powers of 2 and higher, might result in error_huge() trap */
 		do {
@@ -234,7 +276,6 @@ do_quick:
 		} while (uf_tsimp(equation, np));
 		blt(jmp_save, save_save, sizeof(jmp_save));
 	}
-	partial_flag = false;
 	simpb_side(equation, np, true, 1);
 	debug_string(1, "Simplify result before applying polynomial operations:");
 	side_debug(1, equation, *np);
@@ -260,7 +301,7 @@ do_quick:
 		}
 		break;
 	}
-	partial_flag = true;
+	simpb_side(equation, np, true, 1);
 	simp_constant_power(equation, np);
 	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
 	unsimp_power(equation, np);
@@ -325,7 +366,9 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 	uf_allpower(equation, np);
 	last_v = 0;
 	for (vc = 0; vc < ARR_CNT(va); vc++) {
-		for (i = 0, cnt = 0, v1 = -1; i < *np; i += 2) {
+		cnt = 0;
+		v1 = -1;
+		for (i = 0; i < *np; i += 2) {
 			if (equation[i].kind == VARIABLE && equation[i].token.variable > last_v) {
 				if (v1 == -1 || equation[i].token.variable < v1) {
 					v1 = equation[i].token.variable;
@@ -361,30 +404,6 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 	simp_ssub(equation, np, MATCH_ANY, 0.0, power_flag, true, fc_level);
 }
 
-/*
- * The most basic simplification loop.
- * Just does constant simplification.
- */
-elim_loop(equation, np)
-token_type	*equation;	/* pointer to the beginning of equation side to simplify */
-int		*np;		/* pointer to length of equation side */
-{
-	side_debug(6, equation, *np);
-	do {
-		do {
-			do {
-				organize(equation, np);
-			} while (combine_constants(equation, np, true));
-		} while (elim_k(equation, np));
-	} while (simp_pp(equation, np));
-	if (reorder(equation, np)) {
-		do {
-			organize(equation, np);
-		} while (elim_k(equation, np));
-	}
-	side_debug(5, equation, *np);
-}
-
 /*
  * Configurable high level simplify routine.
  */
@@ -504,7 +523,7 @@ int		fc_level;
 
 /*
  * Fix up levels of parentheses in an equation side.
- * Put similar operators on the same level.
+ * Remove unnecessary parentheses and put similar operators on the same level.
  *
  * This is the inner-most loop in Mathomatic, make it fast.
  */
@@ -517,7 +536,7 @@ int		*np;		/* pointer to length of equation side */
 		longjmp(jmp_save, 13);
 	}
 	if (*np > n_tokens) {
-		error("Internal error: Expression array overflow detected.");
+		error("Internal error: Unchecked expression array overflow detected.");
 		longjmp(jmp_save, 13);
 	}
 	org_recurse(equation, np, 0, 1, NULL);
@@ -983,6 +1002,7 @@ double	k2;	/* operand 2 */
 #if	false	/* If fmod() always works.  It doesn't. */
 		*k1p = fmod(*k1p, k2);
 #else
+		/* Another way to get the remainder of division: */
 		*k1p = modf(*k1p / k2, &d) * k2;
 #endif
 		if (true_modulus && *k1p < 0.0) {
@@ -1183,16 +1203,14 @@ not_imaginary:
 }
 
 /*
- * Eliminate operations that involve constants that can be simplified.
- * Fix addition/subtraction of negative or infinity constants.
- * Replace division by a constant with times its reciprocal.
+ * Eliminate or fix operations involving constants that can be simplified or normalized.
  *
- * Return true if one or more eliminations occurred.
+ * Return true if equation side was significantly modified.
  */
 int
 elim_k(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
 {
 	token_type	*p1, *p2, *p3, *p4;
 	token_type	*ep;			/* end pointer */
@@ -1213,6 +1231,7 @@ int		*np;
 		switch (p1->token.operatr) {
 		case PLUS:
 		case MINUS:
+/* Fix addition/subtraction of negative, zero, or infinity constants. */
 			p2 = p1 + 1;
 			if (p1 + 2 < ep && (p1 + 2)->level == level + 1
 			    && ((p1 + 2)->token.operatr == TIMES || (p1 + 2)->token.operatr == DIVIDE)
@@ -1248,16 +1267,14 @@ int		*np;
 						p3++;
 						break;
 					}
-					if (p3->kind == CONSTANT && p3 != p2
-					    && !isfinite(p3->token.constant)) {
+					if (p3->kind == CONSTANT && p3 != p2 && !isfinite(p3->token.constant)) {
 						flag = true;
 					}
 					if (p3 == equation)
 						break;
 				}
 				for (p4 = p1; p4 < ep && p4->level >= level; p4++) {
-					if (p4->kind == CONSTANT && p4 != p2
-					    && !isfinite(p4->token.constant)) {
+					if (p4->kind == CONSTANT && p4 != p2 && !isfinite(p4->token.constant)) {
 						flag = true;
 					}
 				}
@@ -1275,11 +1292,11 @@ int		*np;
 			}
 			break;
 		}
+		p2 = p1 - 1;
 		switch (p1->token.operatr) {
 		case PLUS:
-			p2 = p1 - 1;
-			if (p2->level == level && p2->kind == CONSTANT
-			    && p2->token.constant == 0.0) {
+/* remove 0+ */
+			if (p2->level == level && p2->kind == CONSTANT && p2->token.constant == 0.0) {
 				blt(p2, p1 + 1, (char *) ep - (char *) (p1 + 1));
 				*np -= 2;
 				modified = true;
@@ -1287,11 +1304,9 @@ int		*np;
 			}
 			break;
 		case MINUS:
-			p2 = p1 - 1;
-			if (p2->level == level && p2->kind == CONSTANT
-			    && p2->token.constant == 0.0) {
-				if (p2 == equation
-				    || (p2 - 1)->level < level) {
+/* 0-x to (-1*x) */
+			if (p2->level == level && p2->kind == CONSTANT && p2->token.constant == 0.0) {
+				if (p2 == equation || (p2 - 1)->level < level) {
 					p2->token.constant = -1.0;
 					p1->token.operatr = TIMES;
 					binary_parenthesize(equation, *np, p1 - equation);
@@ -1301,9 +1316,9 @@ int		*np;
 			}
 			break;
 		case TIMES:
-			if ((p1 - 1)->level == level && (p1 - 1)->kind == CONSTANT) {
-				d = (p1 - 1)->token.constant;
-				if (d == 0.0) {
+			if (p2->level == level && p2->kind == CONSTANT) {
+				if (p2->token.constant == 0.0) {
+/* Replace 0*x with 0. */
 					for (p2 = p1 + 2; p2 < ep; p2 += 2) {
 						if (p2->level < level)
 							break;
@@ -1313,20 +1328,23 @@ int		*np;
 					modified = true;
 					continue;
 				}
-				if (fabs(d - 1.0) <= epsilon) {
-					blt(p1 - 1, p1 + 1, (char *) ep - (char *) (p1 + 1));
+				if (fabs(p2->token.constant - 1.0) <= epsilon) {
+/* Replace 1*x with x. */
+					blt(p2, p1 + 1, (char *) ep - (char *) (p1 + 1));
 					*np -= 2;
 					modified = true;
 					continue;
 				}
 			}
 			if ((p1 + 1)->level == level && (p1 + 1)->kind == CONSTANT) {
+/* Move any constant to the beginning of a multiplicative subexpression. */
 				d = (p1 + 1)->token.constant;
 				for (p2 = p1 - 1; p2 > equation; p2--) {
 					if ((p2 - 1)->level < level)
 						break;
 				}
 				if (p2->level == level && p2->kind == CONSTANT) {
+					/* already a constant at the beginning, so don't move anything */
 					break;
 				}
 				blt(p2 + 2, p2, (char *) p1 - (char *) p2);
@@ -1346,10 +1364,9 @@ int		*np;
 			}
 			break;
 		case DIVIDE:
-			p2 = p1 - 1;
-			if (p2->level == level && p2->kind == CONSTANT
-			    && p2->token.constant == 0.0) {
-				for (p2 = p1 + 2; p2 < ep; p2++) {
+			if (p2->level == level && p2->kind == CONSTANT && p2->token.constant == 0.0) {
+/* Replace 0/x with 0. */
+				for (p2 = p1 + 2; p2 < ep; p2 += 2) {
 					if (p2->level < level)
 						break;
 				}
@@ -1360,22 +1377,22 @@ int		*np;
 			}
 			p2 = p1 + 1;
 			if (p2->level == level && p2->kind == CONSTANT) {
+/* Replace division by a constant with times its reciprocal. */
 				f_to_fraction(p2->token.constant, &numerator, &denominator);
 				p2->token.constant = denominator / numerator;
 				p1->token.operatr = TIMES;
 				continue;
 			}
-			if (p2->level == level
-			    && p2->kind == VARIABLE && (p2->token.variable & VAR_MASK) == SIGN) {
+			if (p2->level == level && p2->kind == VARIABLE && (p2->token.variable & VAR_MASK) == SIGN) {
+/* Replace division by a sign variable with times the sign variable. */
 				p1->token.operatr = TIMES;
 				continue;
 			}
 			break;
 		case MODULUS:
-			p2 = p1 - 1;
-			if (p2->level == level && p2->kind == CONSTANT
-			    && p2->token.constant == 0.0) {
-				for (p2 = p1 + 2; p2 < ep; p2++) {
+			if (p2->level == level && p2->kind == CONSTANT && p2->token.constant == 0.0) {
+/* Replace 0%x with 0. */
+				for (p2 = p1 + 2; p2 < ep; p2 += 2) {
 					if (p2->level < level)
 						break;
 				}
@@ -1386,9 +1403,10 @@ int		*np;
 			}
 			break;
 		case POWER:
-			if ((p1 - 1)->level == level && (p1 - 1)->kind == CONSTANT) {
-				if ((p1 - 1)->token.constant == 1.0) {
-					for (p2 = p1 + 2; p2 < ep; p2++) {
+			if (p2->level == level && p2->kind == CONSTANT) {
+				if (p2->token.constant == 1.0) {
+/* Replace 1^x with 1. */
+					for (p2 = p1 + 2; p2 < ep; p2 += 2) {
 						if (p2->level <= level)
 							break;
 					}
@@ -1398,8 +1416,10 @@ int		*np;
 					continue;
 				}
 			}
-			if ((p1 + 1)->level == level && (p1 + 1)->kind == CONSTANT) {
-				if ((p1 + 1)->token.constant == 0.0) {
+			p2 = p1 + 1;
+			if (p2->level == level && p2->kind == CONSTANT) {
+				if (p2->token.constant == 0.0) {
+/* Replace x^0 with 1. */
 					for (p2 = p1 - 1; p2 > equation; p2--) {
 						if ((p2 - 1)->level <= level)
 							break;
@@ -1411,7 +1431,8 @@ int		*np;
 					modified = true;
 					continue;
 				}
-				if (fabs((p1 + 1)->token.constant - 1.0) <= epsilon) {
+				if (fabs(p2->token.constant - 1.0) <= epsilon) {
+/* Replace x^1 with x. */
 					blt(p1, p1 + 2, (char *) ep - (char *) (p1 + 2));
 					*np -= 2;
 					modified = true;
@@ -1909,7 +1930,7 @@ int		*np;
 }
 
 /*
- * Convert 1/m*(-1*b+y) to (y-b)/m.
+ * Convert 1/m*(-1*b+y) to (y-b)/m in equation side.
  *
  * Returns true if equation side was modified.
  * "elim_k()" should be called after this if it returns true.
@@ -1950,6 +1971,9 @@ int		i1, i2;
 	blt(&equation[e2-n1], scratch, n1 * sizeof(token_type));
 }
 
+/*
+ * Recursively check and possibly reorder a "level" subexpression which starts at "loc" in an equation side.
+ */
 static int
 order_recurse(equation, np, loc, level)
 token_type	*equation;
@@ -1959,6 +1983,7 @@ int		*np, loc, level;
 	int	op = 0;
 	int	modified = false;
 
+/* check passed subexpression for validity: */
 	if ((loc & 1) != 0) {
 		goto corrupt;
 	}
@@ -1980,13 +2005,16 @@ int		*np, loc, level;
 				goto corrupt;
 			}
 			if (equation[i].kind == OPERATOR) {
+				if (!equation[i].token.operatr) {
+					goto corrupt;
+				}
 				if (op) {
 					switch (equation[i].token.operatr) {
 					case PLUS:
 					case MINUS:
 						if (op == PLUS || op == MINUS)
 							break;
-						op = 0;
+						goto corrupt;
 					case TIMES:
 					case DIVIDE:
 						if (op == TIMES || op == DIVIDE)
@@ -2001,6 +2029,10 @@ int		*np, loc, level;
 		}
 		i++;
 	}
+	if ((i & 1) != 1) {	/* "i" should be at the end of the current subexpression */
+		goto corrupt;
+	}
+/* reorder subexpression if needed: */
 	switch (op) {
 	case PLUS:
 	case MINUS:
