@@ -1,7 +1,7 @@
 /*
  * Group and combine denominators.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -74,9 +74,9 @@ int		*np, loc, level;
 }
 
 /*
- * Group denominators together.
- * Grouping here means converting "a/b/c/d*e" to "a*e/(b*c*d)" or "a/(b*c*d)*e",
- * not guaranteed to put the grouped divisors last, reorder() puts divisors last.
+ * Group denominators together in an equation side.
+ * Grouping here means converting "a/b/c/d*e" to "a*e/(b*c*d)" or "a/(b*c*d)*e".
+ * Not guaranteed to put the grouped divisors last, reorder() puts divisors last.
  */
 void
 group_proc(equation, np)
@@ -91,7 +91,7 @@ int		*np;
  */
 void
 make_fractions_and_group(n)
-int	n;
+int	n;	/* equation space number */
 {
 	if (n_lhs[n] <= 0)
 		return;
@@ -107,10 +107,25 @@ int	n;
 }
 
 /*
- * Combine denominators in equation side.
+ * Make equation side ready for display.
+ */
+void
+es_make_fractions_and_group(equation, np)
+token_type	*equation;
+int		*np;
+{
+	if (display2d) {
+		elim_loop(equation, np);
+		make_fractions(equation, np);
+		group_proc(equation, np);
+	}
+}
+
+/*
+ * Combine fractions by making the denominators all the same.
  * This means converting "a/b+c/d" to "(a*d+c*b)/b/d".
  *
- * If start_flag is 0, only combine denominators to simplify complex fractions.
+ * If start_flag is 0, only combine denominators to convert complex fractions to simple fractions.
  *
  * If start_flag is 1, always combine denominators.
  *
